@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -56,6 +56,7 @@ class _QtFIFOReader(QObject):
         # can add O_NONBLOCK.
         # pylint: disable=no-member,useless-suppression
         fd = os.open(filepath, os.O_RDWR | os.O_NONBLOCK)
+        # pylint: enable=no-member,useless-suppression
         self._fifo = os.fdopen(fd, 'r')
         self._notifier = QSocketNotifier(fd, QSocketNotifier.Read, self)
         self._notifier.activated.connect(self.read_line)
@@ -247,6 +248,7 @@ class _POSIXUserscriptRunner(_BaseUserscriptRunner):
                                              dir=standarddir.runtime())
             # pylint: disable=no-member,useless-suppression
             os.mkfifo(self._filepath)
+            # pylint: enable=no-member,useless-suppression
         except OSError as e:
             message.error("Error while creating FIFO: {}".format(e))
             return
@@ -444,3 +446,4 @@ def run_async(tab, cmd, *args, win_id, env, verbose=False):
     runner.prepare_run(cmd_path, *args, env=env, verbose=verbose)
     tab.dump_async(runner.store_html)
     tab.dump_async(runner.store_text, plain=True)
+    return runner
