@@ -182,9 +182,16 @@ def init_log(args):
     root = logging.getLogger()
     global console_filter
     if console is not None:
-        console_filter = LogFilter(None)
-        if args.logfilter is not None:
-            console_filter.names = args.logfilter.split(',')
+        if not args.logfilter:
+            negate = False
+            names = None
+        elif args.logfilter.startswith('!'):
+            negate = True
+            names = args.logfilter[1:].split(',')
+        else:
+            negate = False
+            names = args.logfilter.split(',')
+        console_filter = LogFilter(names, negate)
         console.addFilter(console_filter)
         root.addHandler(console)
     if ram is not None:
@@ -202,6 +209,11 @@ def _init_py_warnings():
     """Initialize Python warning handling."""
     warnings.simplefilter('default')
     warnings.filterwarnings('ignore', module='pdb', category=ResourceWarning)
+    # This happens in many qutebrowser dependencies...
+    warnings.filterwarnings('ignore', category=DeprecationWarning,
+                            message="Using or importing the ABCs from "
+                            "'collections' instead of from 'collections.abc' "
+                            "is deprecated, and in 3.8 it will stop working")
 
 
 @contextlib.contextmanager
@@ -498,12 +510,14 @@ class LogFilter(logging.Filter):
     comma-separated list instead.
 
     Attributes:
-        _names: A list of names that should be logged.
+        names: A list of record names to filter.
+        negated: Whether names is a list of records to log or to suppress.
     """
 
-    def __init__(self, names):
+    def __init__(self, names, negate=False):
         super().__init__()
         self.names = names
+        self.negated = negate
 
     def filter(self, record):
         """Determine if the specified record is to be logged."""
@@ -514,12 +528,12 @@ class LogFilter(logging.Filter):
             return True
         for name in self.names:
             if record.name == name:
-                return True
+                return not self.negated
             elif not record.name.startswith(name):
                 continue
             elif record.name[len(name)] == '.':
-                return True
-        return False
+                return not self.negated
+        return self.negated
 
 
 class RAMHandler(logging.Handler):
