@@ -310,7 +310,7 @@ def _chromium_version():
     """Get the Chromium version for QtWebEngine.
 
     This can also be checked by looking at this file with the right Qt tag:
-    https://github.com/qt/qtwebengine/blob/dev/tools/scripts/version_resolver.py#L41
+    http://code.qt.io/cgit/qt/qtwebengine.git/tree/tools/scripts/version_resolver.py#n41
 
     Quick reference:
     Qt 5.7:  Chromium 49
@@ -318,7 +318,7 @@ def _chromium_version():
     Qt 5.9:  Chromium 56
     Qt 5.10: Chromium 61
     Qt 5.11: Chromium 65
-    Qt 5.12: Chromium 69 (?)
+    Qt 5.12: Chromium 69 (? - current dev branch: 67)
 
     Also see https://www.chromium.org/developers/calendar
     """
@@ -399,6 +399,7 @@ def version():
     lines += [
         'Frozen: {}'.format(hasattr(sys, 'frozen')),
         "Imported from {}".format(importpath),
+        "Using Python from {}".format(sys.executable),
         "Qt library executable path: {}, data path: {}".format(
             QLibraryInfo.location(QLibraryInfo.LibraryExecutablesPath),
             QLibraryInfo.location(QLibraryInfo.DataPath)
